/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.cometd.bayeux;

/**
 * Data Filter<br/>
 * Data filters are used to transform data as it is sent to a Channel.
 * Messages are filtered as the message is published to a channel, invoking the
 * {@link Channel#publish(Message)} method.<br/>
 * This method gets invoked in two different scenarios, the first being when a message is received from
 * a remote client, and the Bayeux implementation invokes the publish method directly.
 * The second scenario is when a local client invokes {@link Channel#publish(Message)} directly in the local JVM.
 * @author Greg Wilkins
 *
 */
public interface DataFilter
{
    /**
     * Runs a message through the filter. Filtering can only modify an existing object, it can not replace it.
     * @param data Message - the message to be filtered, may not be null
     */
    public void filter(Message data);
}
